/**
 * Kalender-Applet
 */
import java.awt.*;
import java.applet.Applet;
import java.util.*;
import java.text.NumberFormat;

public class Kalender extends Applet
{
   /**
    * Pixel je Anzeige-Zeile
    */
   private final static int zeiPix = 20 ;

   /**
    * Pixel je Anzeige-Spalte
    */
   private final static int spaPix = 20 ;

   /**
    * Pixel je Anzeige-Spalte
    */
   private final static int lRand = 5 ;

   /**
    * angezeigtes Datum
    */
   private Date bezugDate;

   /**
    * Display-Zeile
    */
   private int displayZeile ;

   /**
    * Display-Spalte
    */
   private int displaySpalte ;

   /**
    * Init-Methode des Applets
    */
   public void init()
   {
      bezugDate = new Date() ;
      repaint() ;
   }

   /**
    * paint-Methode zur Aktualisierung der Anzeige
    */
   public void paint( Graphics g )
   {
      int zei = 4 ;
      int i , j , ii ;
      Date iDate ; // temporrer Datumsmerker fr die Anzeige der Kalenderwochen

      // Laufvariable zum Anzeige Tageszahlen
      // zum ersten des Monats gehen und dann auf Montag normalisieren
      Date zeigDate = normalizeDayOnMonday( firstDayOfMonth( bezugDate ) ) ;

      String[] wtag = { "KW" , "Mo" , "Di" , "Mi" , "Do" , "Fr" , "Sa" , "So" } ;

      //              zei spa
      stringPaint( g , 1 , 0 , nameDayOfWeek( bezugDate ) + " " + dateToString( bezugDate ) ) ;
      iDate = zeigDate ;
      for( i = 0 ; i <= 7 ; i++ ) // Anzeige Wochentagsabkrzungen
      {
         stringPaint( g , 3 , ( i * 2 ) , wtag[ i ] ) ;
      }

      // horizontale Trennlinie
      int lineZei = ( 3 * zeiPix ) + 5 ;
      g.drawLine( 0 , lineZei , 310 , lineZei ) ;

      // vertikale Trennlinie
      int lineSpa = zeiPix + ( zeiPix / 2 ) ;
      g.drawLine( lineSpa , zeiPix * 2 , lineSpa , 310 ) ;

      for( i = 1 ; i <= 6 ; i++ ) // Anzeige der Kalenderwochen
      {
         j = week( iDate ) ;
         if ( i <= 1 && j > 52 ) {
           j = 1 ; // oberste Zeile, Kalenderwoche wird am Jahresanfang von 53 auf 1 gesetzt
         }
         else if ( i > 4 && j < 2 ) {
           j = 53 ; // unterste Zeile, Kalenderwoche wird am Jahresende von 1 auf 53 gesetzt
         }
         stringPaint( g , i + 3 , 0 , "" + j ) ; // Anzeige Kalenderwoche als Zahl
         iDate = addDays( iDate , 7 ) ; // eine Woche weiterschalten
      }
      for( i = 1 ; i <= 6 ; i++ ) // Anzeige Tage im Monat als Zahl
      {
         j = zeigDate.getDay() ; // Tag im Monat
         for( ii = 1 ; ii <= 7 ; ii++ )
         {
            if ( zeigDate.getMonth() == bezugDate.getMonth() )
            {
               stringPaint( g , zei , ( j * 2 ) , stringZero( zeigDate.getDate() , 2 ) ) ; // nicht im aktuellen Monat liegende Tage werden anders angezeigt
            }
            else
            {
               g.setColor( Color.blue ) ;
               stringPaint( g , zei , ( j * 2 ) , stringZero( zeigDate.getDate() , 2 ) ) ;
               g.setColor( Color.black ) ;
            } // end if
            if ( dateToString( bezugDate ).equals( dateToString( zeigDate ) ) ) // Anleuchtposition
            {
               // g.drawRect(((j*2)*20)-2,((zei-1)*20)+7,18,15);
               g.drawRect( ( ( j * 2 ) * spaPix ) - 2 + lRand , ( ( zei - 1 ) * zeiPix ) + 7 , 17 , 16 ) ;
            }
            j++ ;
            zeigDate = dateInc( zeigDate ) ;
         } // next
         zei++ ;
      } // next
   } // end method paint

   /**
    * interne String-Paint-Methode<BR>
    * Diese Methode wandelt die Zeichenposition in Pixelposition um.<BR>
    *
    * @param g GraphicContext
    * @param i Zeile
    * @param j Spalte
    * @param s auszugebender String
    */
   private void stringPaint( Graphics g , int i , int j , String s )
   {
      char separated[] =  new char [ s.length() ] ;
      s.getChars( 0 , s.length() , separated , 0 ) ;
      g.drawChars( separated , 0 , s.length() , ( j * zeiPix ) + lRand , i * spaPix ) ;
   } // end method stringPaint

   /**
    * Methode zum Abarbeiten der Tastendrcke
    */
   public boolean keyDown( Event evt , int Key )
   {
      if( Key == 1004 ) // Pfeil-hoch
      {
         bezugDate = addDays( bezugDate , -7 ) ;
         displayZeile-- ;
      }
      else if( Key == 1005 ) // Pfeil runter
      {
         bezugDate = addDays( bezugDate , 7 ) ;
         displayZeile++ ;
      }
      else if( Key == 1007 ) // Pfeil-rechts
      {
         bezugDate = dateInc( bezugDate ) ;
         displaySpalte++ ;
         if ( displaySpalte > 7 )
         {
            displaySpalte=1 ;
            displayZeile++ ;
         }
      }
      else if( Key == 1006 ) // Pfeil-links
      {
         bezugDate = dateDec( bezugDate ) ;
         displaySpalte-- ;
         if( displaySpalte < 1 )
         {
            displaySpalte = 7 ;
            displayZeile-- ;
         }
      }
      else if( Key == 1000 ) // Home (Pos1)
      {
         bezugDate = normalizeDayOnMonday( bezugDate ) ;
         displaySpalte = 1 ;
      }
      else if( Key == 1001 ) // End
      {
         bezugDate = normalizeDayOnMonday( bezugDate ) ;
         bezugDate = addDays( bezugDate , 6 ) ;
         displaySpalte = 7 ;
      }
      else if( Key == 1002 ) // PgUp (Bild hoch)
      {
         bezugDate = addDays( bezugDate , -28 ) ;
         displayZeile -= 4 ;
      }
      else if( Key == 1003 ) // PgDn (Bild runter)
      {
         bezugDate=addDays( bezugDate , 28 ) ;
         displayZeile += 4 ;
      }
      repaint() ;
      return true ;
   } // end method keyDown

   /**
    * Behandlung Maustaste
    */
   public boolean mouseDown( Event evt , int x , int y )
   {
      int zei = 4 ;
      int i , j , ii ;
      Date zeigDate = normalizeDayOnMonday( firstDayOfMonth( bezugDate ) ) ; // zum ersten des Monats gehen und dann auf Montag normalisieren
      // Feststellen auf welchen Tag geklickt wurde
      for( i = 1 ; i <= 6 ; i++ )
      {
         j = zeigDate.getDay() ; // Tag im Monat
         for( ii = 1 ; ii <= 7 ; ii++ )
         {

            if ( ( x >= ( ( j * 2 ) * spaPix ) - 2 + lRand         )
              && ( x <= ( ( j * 2 ) * spaPix ) - 2 + lRand + 17    )
              && ( y >= ( ( zei - 1 ) * zeiPix ) + 7               )
              && ( y <= ( ( zei - 1 ) * zeiPix ) + 7 + 17          ) )
            {
               bezugDate = zeigDate ; // neues Datum setzen
               repaint() ;
               return true ;
            }
            j++ ;
            zeigDate = dateInc( zeigDate ) ;
         } // next
         zei++ ;
      } // next

      repaint() ;
      return true ;
   } // end method mouseDown

   /**
    * Datum incrementieren
    */
   private Date dateInc( Date d )
   {
      return new Date( d.getTime() + 86400000 ) ;
   } // end method dateInc

   /**
    * Datum decrementieren
    */
   private Date dateDec( Date d )
   { // Datum decrementieren
      return new Date( d.getTime() - 86400000 ) ;
   } // end method dateDec

  /**
   * int-Wert in String mit fhrenden Nullen umwandeln
   */
   private String stringZero( int nr , int digits )
   {
      String t = "" + nr ;
      while( t.length() < digits )
      {
         t = "0" + t ;
      }
      return t ;
   } // end method stringZero

   /**
    * First of Year<BR>
    * erster Tag des Jahres
    */
   private Date firstDayOfYear( Date tag )
   {
      Date td = new Date( tag.getTime() ) ; // Kopie des Parameters anlegen um Vernderung des Wertes (Seiteneffekt) zu vermeiden
      td.setMonth( 0 ) ;
      td.setDate( 1 ) ;
      td.setHours( 12 ) ;
      td.setMinutes( 0 ) ;
      td.setSeconds( 0 ) ;
      return td;
   } // end method firstDayOfYear

  /**
   * Day of Week<BR>
   * Wochentag<BR>
   * Sonntag == 0<BR>
   * Samstag == 6<BR>
   */
   private int DayOfWeek( Date tag )
   {
      return tag.getDay() ;
   } // end method DayOfWeek

   /**
    * Name Day of Week<BR>
    * Name des Wochentages als String<BR>
    * Sonntag == 0<BR>
    * Samstag == 6<BR>
    */
   private String nameDayOfWeek( Date tag )
   {
      String[] acdow = { "Sonntag" , "Montag" , "Dienstag" , "Mittwoch" , "Donnerstag" , "Freitag" , "Samstag" } ;
      return acdow[ tag.getDay() ] ;
   } // end method nameDayOfWeek

   /**
    * Day of Year<BR>
    * Tag im Jahr<BR>
    */
   private int DayOfYear( Date tag )
   {
      Date td = new Date( tag.getTime() ) ;
      long t1 , t2 ;
      t1 = td.getTime() / ( 24 * 60 * 60 * 1000 ) ;
      t2 = firstDayOfYear( td ).getTime() / ( 24 * 60 * 60 * 1000 ) ;
      return ( int ) ( t1 - t2 ) ;
   } // end method DayOfYear

   /**
    * First of Month<bR>
    * erster Tag des Monates<BR>
    */
   private Date firstDayOfMonth( Date tag )
   {
      Date td = new Date(tag.getTime());
      td.setDate(1);
      return td;
   } // end method firstDayOfMonth

   /**
    * Datum auf Montag der Woche normalisieren
    */
   private Date normalizeDayOnMonday( Date tag )
   {
      int t = tag.getDay() ;
      long t1 ;
      Date t2 ;
      t1 = tag.getTime() ;
      if ( t < 1 )
      {
         t1 = t1 - ( 6 * 86400000 ) ;
      }
      else
      {
         t-- ;
         t1 = t1 - ( t * 86400000 ) ;
      }
      t2 = new Date( t1 ) ;
      t2.setHours( 12 ) ; // normalisieren auf Mittag
      t2.setMinutes( 0 ) ;
      return t2 ;
   } // end method normalizeDayOnMonday

   /**
    * Datum auf Mittag normalisieren<BR>
    */
   private Date ddnorm(Date tag) {
    Date t=new Date(tag.getTime());
    t.setHours(12); // normalisieren auf Mittag
    t.setMinutes(0);
    return t;
  }

   /**
    * Anzahl Tage zum Datum Addieren
    *
    * @param d        zu erhhendes Datum
    * @param addtagte Anzahl Tage, um die erhht werden soll
    */
   private Date addDays( Date d , int addtage )
   {
      Date t = new Date( d.getTime() ) ;
      long t1,t2;
      t.setHours( 12 ) ; // normalisieren auf Mittag
      t.setMinutes( 0 ) ;
      t1 = t.getTime() ;
      t2 = ( (long) addtage * 86400000 ) ;
      t1 = t1 + t2 ;
      t.setTime( t1 ) ;
      return t ;
   } // end method addDays

   /**
    * Woche im Jahr
    */
   private int week( Date tag )
   {
      int iReturn ;
      Date td = new Date( tag.getTime() ) ; // Kopie des Paramters erstellen, um Seiteneffekte zu vermeiden
      iReturn = (int) ( DayOfYear( td ) / 7 ) + 2 ;
      if ( iReturn > 52 )
      {
         iReturn = 1;
      }
      return iReturn ;
   } // end method week

  /**
   * Datum in String umwandeln
   */
  private String dateToString( Date tag )
  {
    return stringZero(tag.getDate(),2)+"."+stringZero(tag.getMonth()+1,2)+"."+stringZero(tag.getYear()+1900,4);
  }
}
